##########################################################
###################  Input ###############################
##########################################################
library("matrixStats")
dir.output<-getwd()  # set the directory to save the output
setwd(dir.output)

#### matrix.type = "Expo" means AR(1) correlation structure and corresponds to Model 1 in Figure1.
#### matrix.type = "Poly" means polynomial decay and corresponds to Model 2 in Figure1.
#### matrix.type = "SiG"  means singular matrices and corresponds to Model 3 in Figure1.
matrix.type<-"SiG"

#### dist.type = "normal" corresponds to the result for normal distribution in Figure 1 in the main text
#### dist.type = "t4"  corresponds to the result for t(4) in Figure 1 in the supplemental text.
dist.type<-"normal"


##########################################################
###################  Functions ###########################
##########################################################

Cauchy<-function(Pval){
    colMeans(tan((0.5-Pval)*pi))
}

Sigma.chol.generate<-function(matrix.type,p,para){
    if (matrix.type=="Expo"){
        rho<-para
        Sigma<-matrix(0,nrow = p,ncol = p)
        for (i in 1:p){
            for (j in 1:p){
                Sigma[i,j]<-rho^{abs(i-j)}
            }
        }
    }
    
    if (matrix.type=="Poly"){
        r<-para
        Sigma<-matrix(0,nrow = p,ncol = p)
        for (i in 1:p){
            for (j in 1:p){
                if (i==j){
                    Sigma[i,j]<-1
                }else{
                    Sigma[i,j]<-1/(0.7+abs(i-j)^r)
                }
            }
        }
    }
    
    if (matrix.type=="SiG"){
        d<-para
        A<-matrix(NA,nrow=(p/5),ncol=p)
        for (i in 1:(p/5)){
            for (j in 1:p){
                A[i,j]<-1/(i-j+d)
            }
        }
        A<-A%*%diag(1/sqrt(colSums(A^2)))
        Sigma.chol<-t(A)
    }
    
    #### chol decomposition
    if (matrix.type!="SiG"){
        #suppressWarnings(Sigma.chol<-chol(Sigma,pivot = TRUE))
        Sigma.chol<-chol(Sigma,pivot = TRUE)
        Sigma.chol<-Sigma.chol[,order(attr(Sigma.chol, "pivot"))]
        Sigma.chol<-t(Sigma.chol)
    }
    
    return(Sigma.chol)
}

Run<-function(B,Bi,Sigma.chol,dist.type,k,file.num){
    Stat.c<-c()
    for (ii in 1:(B/Bi)){
        p<-dim(Sigma.chol)[2]
        X<-rnorm(p*Bi)
        dim(X)<-c(p,Bi)
        X<-Sigma.chol%*%X
        
        if (dist.type=="normal"){
            Pval<-(1-pnorm(abs(X)))*2
        }else if (dist.type=="t4"){
            denomin<-sqrt(rchisq(Bi,df=4)/4)
            for (jj in 1:Bi){
                X[,jj]<-X[,jj]/denomin[jj]
            }
            Pval<-(1-pt(abs(X),df=4))*2
        }
        
        stat.c<-Cauchy(Pval)
        Stat.c<-c(Stat.c,stat.c)
    }
    
    save(Stat.c,file = paste("Sample_cauchy",k,"_",file.num,".Rdata",sep=""))
    rm(Stat.c)
    gc()
}


##########################################################
####################  Run  ###############################
##########################################################

B<-10^7
Bi<-10^5
p.pool<-c(5,20,50,100,300,500)

### Exponential decay
rho.pool<-c(0.2,0.4,0.6,0.8,0.99)

### Polynomial decay
r.pool<-c(0.5,1,1.5,2,2.5)


### Degenerate matrix
d.pool<-c(0.1,0.3,0.5,0.7,0.9)

### Go
if (matrix.type=="Expo"){
    para.pool<-rho.pool
}else if (matrix.type=="Poly"){
    para.pool<-r.pool
}else if (matrix.type=="SiG"){
    para.pool<-d.pool
}


for (file.num in 1:10){
    set.seed(58143*file.num)
    
    k<-1
    for (p in p.pool){
        for (para in para.pool){
            Sigma.chol<-Sigma.chol.generate(matrix.type,p,para)
            Run(B,Bi,Sigma.chol,dist.type,k,file.num)
            k<-k+1
        }
    }
}

