##########################################################
###################  Input ###############################
##########################################################
library("matrixStats")
dir.output<-getwd()  # set the directory to save the output
setwd(dir.output)

### set CV.type.CCT = "permu" to get Figure 3 in the main text
### set CV.type.CCT = "analytic" to get Figure 2 in the supplemental materials. 
CV.type.CCT<-"permu"    # "analytic","permu"

#######################################################
################ Functions ############################
#######################################################

HC<-function(Pval){
    p<-dim(Pval)[1]
    B<-dim(Pval)[2]
    
    Pval.sort<-matrix(0,nrow=p,ncol=B)
    if (p<=10){
        for(i in 1:p){
            Pval.sort[i,]<-colOrderStats(Pval,i)
        }
    }else{
        for(i in 1:B){
            Pval.sort[,i]<-sort(Pval[,i])
        }
    }
    Pval<-Pval.sort
    
    HC.stat<-(matrix(rep(c(1:p)/p,B),ncol=B)-Pval)/sqrt(Pval*(1-Pval))
    HC.stat<-colMaxs(HC.stat)*sqrt(p)
    return(HC.stat)
}

BJ<-function(Pval){
    p<-dim(Pval)[1]
    B<-dim(Pval)[2]
    
    Pval.sort<-matrix(0,nrow=p,ncol=B)
    if (p<=10){
        for(i in 1:p){
            Pval.sort[i,]<-colOrderStats(Pval,i)
        }
    }else{
        for(i in 1:B){
            Pval.sort[,i]<-sort(Pval[,i])
        }
    }
    Pval<-Pval.sort
    
    Trt.quantile<-matrix(rep(c(1:p)/p,B),ncol=B)
    BJ.stat<-Trt.quantile*log(Trt.quantile/Pval)
    #### The second part of BJ statistic, which is alwasy 0 when Trt.quantile=1.
    BJ.stat.2<-(1-Trt.quantile)*log((1-Trt.quantile)/(1-Pval))
    BJ.stat.2[p,]<-0
    BJ.stat<-BJ.stat+BJ.stat.2
    
    BJ.stat<-colMaxs(BJ.stat[1:floor(p/2),])*p
    return(BJ.stat)
}

MinP<-function(Pval){
    return(-colMins(Pval))
}

Cauchy<-function(Pval){
    colMeans(1*tan((0.5-Pval)*pi))
}

Get.fun<-function(Method.pool){
    Funs<-list()
    for (test in Method.pool){
        if (test=="HC"){
            Funs[[test]]<-HC
        }else if (test=="BJ"){
            Funs[[test]]<-BJ
        }else if (test=="MinP"){
            Funs[[test]]<-MinP
        }else if (test=="Cauchy"){
            Funs[[test]]<-Cauchy
        }
    }
    return(Funs)
}

#### Sigma is a correlation matrix
Cut<-function(Sigma,N.rep,Method.pool,alpha,block.size=5000){
    N<-N.rep
    B<-c(rep(block.size,N%/%block.size))
    if (N%%block.size>0){
        B<-c(B,N%%block.size)
    }
    
    Funs<-Get.fun(Method.pool)
    
    #### chol decomposition
    suppressWarnings(Sigma.chol<-chol(Sigma,pivot = TRUE))
    Sigma.chol<-Sigma.chol[,order(attr(Sigma.chol, "pivot"))]
    Sigma.chol<-t(Sigma.chol)

    ## dimension
    p<-dim(Sigma)[1]
    
    Stat<-list()
    for (i in 1:length(B)){
        Bi<-B[i]
        X<-rnorm(p*Bi)
        dim(X)<-c(p,Bi)
        X<-Sigma.chol%*%X
        Pval<-(1-pnorm(abs(X)))*2
        
        for (test in Method.pool){
            Stat[[test]]<-c(Stat[[test]],Funs[[test]](Pval))
        }
    }
    
    cut.offs<-c()
    for (test in Method.pool){
        cut.offs<-c(cut.offs,quantile(Stat[[test]],probs = 1-alpha))
    }
    names(cut.offs)<-Method.pool
    return(cut.offs)
}

#### Sigma is a correlation matrix
Power.cal<-function(Sigma,s,N.rep,cut.offs,Signal.type,block.size=5000){
    N<-N.rep
    B<-c(rep(block.size,N%/%block.size))
    if (N%%block.size>0){
        B<-c(B,N%%block.size)
    }
    
    Method.pool<-names(cut.offs)
    Funs<-Get.fun(Method.pool)
    
    #### chol decomposition
    suppressWarnings(Sigma.chol<-chol(Sigma,pivot = TRUE))
    Sigma.chol<-Sigma.chol[,order(attr(Sigma.chol, "pivot"))]
    Sigma.chol<-t(Sigma.chol)
    
    ## dimension
    p<-dim(Sigma)[1]
    
    Stat<-list()
    for (i in 1:length(B)){
        Bi<-B[i]
        X<-rnorm(p*Bi)
        dim(X)<-c(p,Bi)
        X<-Sigma.chol%*%X
        ## generate mu
        MU<-matrix(0,nrow = p,ncol = Bi)
        for (j in 1:Bi){
            if (Signal.type=="constant"){
                MU[sample.int(p,size = s),j]<-sqrt(3*log(p))/s^(1/3)
            }else if (Signal.type=="unif+-"){
                MU[sample.int(p,size = s),j]<-runif(s,min = -sqrt(4*log(p)),max = sqrt(4*log(p)))
            }           
        }
        X<-X+MU # alternative
        ## Pval
        Pval<-(1-pnorm(abs(X)))*2
        
        for (test in Method.pool){
            Stat[[test]]<-c(Stat[[test]],Funs[[test]](Pval))
        }
    }
    
    powers<-rep(NA,length(Method.pool))
    names(powers)<-Method.pool
    for (test in Method.pool){
        powers[test]<-mean(Stat[[test]]>cut.offs[[test]])
    }
    return(powers)
}


####################################################################
################## Power simulation ################################
####################################################################


Method.pool<-c("MinP","HC","BJ","Cauchy")
alpha<-0.05  # significance level
N.cutoff<-10^5  # number of replications for calculating critical values
N.power<-10^4   # number of replications for evaluating power
p.pool<-c(20,40,60) # dimension
s.pool<-matrix(c(c(1,2,4),c(2,4,8),c(3,6,12)),nrow = 3) # #signls
Signal.type<-"constant"    
Sigma.type<-"Exchange"

####################################################################
################## Power simulation ################################
####################################################################
rho.pool<-seq(0,0.4,0.05)

####### get critical values at alpha and save
if (!file.exists(paste("Cutoffs_",alpha,"_",Sigma.type,".Rdata",sep=""))){
    set.seed(38920)
    CVs<-list()
    for (p in p.pool){
        for (rho in rho.pool){
            ## generate Sigma
            Sigma<-matrix(rho,nrow = p,ncol = p)
            diag(Sigma)<-1
            
            cut.offs<-Cut(Sigma,N.rep=N.cutoff,Method.pool,alpha)
            CVs[[as.character(p)]][[as.character(rho)]]<-cut.offs
        }
    }
    save(CVs,file=paste("Cutoffs_",alpha,"_",Sigma.type,".Rdata",sep=""))
}else{
    load(paste("Cutoffs_",alpha,"_",Sigma.type,".Rdata",sep=""))
}

####### calculate power
set.seed(71231)

Power<-list()
for (test in Method.pool){
    Power[[test]]<-list()
    for (p in p.pool){
        Power[[test]][[as.character(p)]]<-matrix(NA,nrow = length(rho.pool),ncol = dim(s.pool)[1])
        colnames(Power[[test]][[as.character(p)]])<-s.pool[,which(p==p.pool)]
    }
}

for (p in p.pool){
    for (rho in rho.pool){
        ## generate Sigma
        Sigma<-matrix(rho,nrow = p,ncol = p)
        diag(Sigma)<-1
        
        cut.offs<-CVs[[as.character(p)]][[as.character(rho)]]
        if (CV.type.CCT=="analytic"){
            cut.offs[["Cauchy"]]<-qcauchy(1-alpha)  ## critical value by Cauchy approximation
        }
        
        for (s in s.pool[,which(p==p.pool)]){
            powers<-Power.cal(Sigma,s,N.rep=N.power,cut.offs,Signal.type)
            for (test in Method.pool){
                Power[[test]][[as.character(p)]][which(rho==rho.pool),as.character(s)]<-powers[test]
            }
        }
    }
    
}

save(Power,file = paste("Power_simu_",Sigma.type,"_",CV.type.CCT,"_",Signal.type,".Rdata",sep=""))

########################################################
#################### plot ##############################
########################################################

library("ggplot2")
result.plot<-data.frame(matrix(NA,nrow = 0,ncol = 5))

for (p in p.pool){
    for (method in Method.pool){
        for (s in s.pool[,which(p==p.pool)]){
            temp<-data.frame(Power[[method]][[as.character(p)]][,as.character(s)],
                        rep(method,length(rho.pool)),
                        rep(as.character(p),length(rho.pool)),
                        rep(paste(as.character(100*s/p),"%",sep=""),length(rho.pool)),
                        rho.pool)
            if (method=="Cauchy"){
                temp[,2]<-rep("CCT",length(rho.pool))
            }
            result.plot<-rbind(result.plot,temp)
        }
    }
}


colnames(result.plot)<-c("power","test","p","s","rho_sq")
result.plot[,"test"] <- factor(result.plot[,"test"], levels = c("CCT", "MinP", "HC","BJ"))

g<-ggplot(result.plot, aes(x=rho_sq, y=power, color=test,linetype=test))+geom_line(size=1)+
    scale_linetype_manual(values=c(1:4))
g<-g+xlab(expression(rho))+facet_grid(s~p,scales="free_x")+theme_bw()
g+guides(linetype=guide_legend(keywidth = 3.8))+theme(legend.title=element_blank(),legend.position="bottom")

